@@ -5,15 +5,15 @@
 
     Implements two-step verification of HOTP/TOTP.
 
-    :copyright: (c) 2013 by Hsiaoming Yang.
+    :copyright: (c) 2013 - 2014 by Hsiaoming Yang.
     :license: BSD, see LICENSE for more details.
 """
-import base64
-import hashlib
-import hmac
-import struct
 import sys
 import time
+import hmac
+import base64
+import struct
+import hashlib
 import warnings
 
 
@@ -28,10 +28,10 @@ else:
 
 __author__ = 'Hsiaoming Yang <me@lepture.com>'
 __homepage__ = 'https://github.com/lepture/otpauth'
-__version__ = '0.2.0'
+__version__ = '0.3.0'
 
 
-__all__ = ['OtpAuth', 'HOTP', 'TOTP']
+__all__ = ['OtpAuth', 'HOTP', 'TOTP', 'generate_hotp', 'generate_totp']
 
 
 HOTP = 'hotp'
@@ -52,29 +52,17 @@ class OtpAuth(object):
 
         :param counter: HOTP is a counter based algorithm.
         """
-        # https://tools.ietf.org/html/rfc4226
-        msg = struct.pack('>Q', counter)
-        digest = hmac.new(to_bytes(self.secret), msg, hashlib.sha1).digest()
+        return generate_hotp(self.secret, counter)
 
-        ob = digest[19]
-        if python_version == 2:
-            ob = ord(ob)
-
-        pos = ob & 15
-        base = struct.unpack('>I', digest[pos:pos + 4])[0] & 0x7fffffff
-        token = base % 1000000
-        return token
-
-    def totp(self, period=30):
+    def totp(self, period=30, timestamp=None):
         """Generate a TOTP code.
 
         A TOTP code is an extension of HOTP algorithm.
 
         :param period: A period that a TOTP code is valid in seconds
+        :param timestamp: Create TOTP at this given timestamp
         """
-        # https://tools.ietf.org/html/rfc6238
-        counter = int(time.time()) // period
-        return self.hotp(counter)
+        return generate_totp(self.secret, period, timestamp)
 
     def valid_hotp(self, code, last=0, trials=100):
         """Valid a HOTP code.
@@ -92,13 +80,14 @@ class OtpAuth(object):
                 return i
         return False
 
-    def valid_totp(self, code, period=30):
+    def valid_totp(self, code, period=30, timestamp=None):
         """Valid a TOTP code.
 
         :param code: A number that is less than 6 characters.
         :param period: A period that a TOTP code is valid in seconds
+        :param timestamp: Validate TOTP at this given timestamp
         """
-        return valid_code(code) and self.totp(period) == int(code)
+        return valid_code(code) and self.totp(period, timestamp) == int(code)
 
     def to_uri(self, type, label, issuer, counter=None):
         """Generate the otpauth protocal string.
@@ -144,6 +133,41 @@ class OtpAuth(object):
         return self.to_uri(type, label, issuer, counter)
 
 
+def generate_hotp(secret, counter=4):
+    """Generate a HOTP code.
+
+    :param secret: A secret token for the authentication.
+    :param counter: HOTP is a counter based algorithm.
+    """
+    # https://tools.ietf.org/html/rfc4226
+    msg = struct.pack('>Q', counter)
+    digest = hmac.new(to_bytes(secret), msg, hashlib.sha1).digest()
+
+    ob = digest[19]
+    if python_version == 2:
+        ob = ord(ob)
+
+    pos = ob & 15
+    base = struct.unpack('>I', digest[pos:pos + 4])[0] & 0x7fffffff
+    token = base % 1000000
+    return token
+
+
+def generate_totp(secret, period=30, timestamp=None):
+    """Generate a TOTP code.
+
+    A TOTP code is an extension of HOTP algorithm.
+
+    :param secret: A secret token for the authentication.
+    :param period: A period that a TOTP code is valid in seconds
+    :param timestamp: Current time stamp.
+    """
+    if timestamp is None:
+        timestamp = time.time()
+    counter = int(timestamp) // period
+    return generate_hotp(secret, counter)
+
+
 def to_bytes(text):
     if isinstance(text, string_type):
         # Python3 str -> bytes
